#ifndef _RTCV_IMAGE_H_
#define _RTCV_IMAGE_H_
//=================================================================================
//=================================================================================
///
/// \file	 rtcvImage.h
///
/// \brief
///			 
///	\par	 Contents
///
/// \author  Jan Salmen
///
/// \date    2009-06-04
///
/// \par     Copyright (c) 2009:
///          Institut fr Neuroinformatik
///          Universitt Bochum
//=================================================================================
//=================================================================================


#include <string.h>
#include <cmath>
#include "assert.h"
#include "rtcvBiLinInterpol.h"


template <typename T>
class rtcvImage
{

public:


	
	rtcvImage<T>(const rtcvImage<T> & img);
	rtcvImage<T>(const int sx = 0, const int sy = 0, const T * pImg = 0);

	~rtcvImage<T>();


	rtcvImage<T> & operator=(const rtcvImage<T> & img);
	
	void changeSize(const int sx, const int sy);
	void setData( const T * pDatNew );

	void fill( const T & val );

	inline T * getDataPointer() const { return pData; }
	inline const T * getDataPointerConst() const { return pData; }

	inline int getSx() const { return sx; }
	inline int getSy() const { return sy; }
	inline int getSz() const { return sx*sy; }

	inline T getValue( const int idx ) const
	{
		assert( (idx >= 0) && (idx < sx*sy) );

		return ( pData[idx] );
	}

	inline T getValue( const int x, const int y ) const
	{
		assert( (x >= 0) && (x < sx) );
		assert( (y >= 0) && (y < sy) );

		return ( pData[y * sx + x] );
	}

	inline void setValue( const int idx, const T & val ) const
	{
		assert( (idx >= 0) && (idx < sx*sy) );

		pData[idx] = val;

		return;
	}

	inline void setValue( const int x, const int y, const T & val ) const
	{
		assert( (x >= 0) && (x < sx) );
		assert( (y >= 0) && (y < sy) );

		pData[y * sx + x] = val;

		return;
	}

	inline T getMean() const
	{
		int sum = 0;  // TODO: oder grerer Datentyp...
		const int sz = sx*sy;

		for ( int i = 0; i < sz; sum += pData[i++] );

		const T mean = (T)floor( 0.5 + sum / sz );

		return mean;
	}
	
private:
	T * pData;

	int sx;
	int sy;
};


template<typename T> rtcvImage<T>::rtcvImage(const rtcvImage<T> & img)
{
	this->pData = 0;
	this->sx = 0;
	this->sy = 0;

	this->changeSize( img.sx, img.sy );
	this->setData( img.getDataPointer() );
}


template<typename T> rtcvImage<T>::rtcvImage(const int sx, const int sy, const T * pImg)
{
	this->pData = 0;
	this->sx = 0;
	this->sy = 0;

	this->changeSize(sx, sy);

	if (pImg) setData( pImg );
}


template<typename T> rtcvImage<T>::~rtcvImage()
{
	delete[] pData;
	pData = 0;
}


template<typename T> rtcvImage<T> & rtcvImage<T>::operator=(const rtcvImage<T> & img)
{
	if ( this != &img )
	{
		this->changeSize( img.sx, img.sy );
		this->setData( img.getDataPointer() );
	}

	return *this;
}

	
template<typename T> void rtcvImage<T>::changeSize(const int sx, const int sy)
{
	assert( (sx >= 0) && (sy >= 0) );

	if ( (sx != this->sx) || (sy != this->sy) )
	{
		delete[] pData;
		pData = 0;
		
		if ( (sx > 0) && (sy > 0) )
		{
			pData = new T[sx*sy];
			this->sx = sx;
			this->sy = sy;
		}
		else
		{
			this->sx = 0;
			this->sy = 0;
		}
	}

	return;
}


template<typename T> void rtcvImage<T>::setData( const T * pDatNew )
{
	assert( ((sx == 0) && (sy == 0)) || (pDatNew != 0) );
	
	memcpy( pData, pDatNew, sx*sy*sizeof(T) );
	
	return;
}


template<typename T> void rtcvImage<T>::fill( const T & val )
{
	memset( getDataPointer(), val, getNoBytes() );

	return;
}


#endif  // #ifndef _RTCV_IMAGE_H_